@@ -1,5 +1,6 @@
 /*
-Copyright (C) 2013-2014 Alessio Garzi <gun101@email.it>
+Copyright (C) 2013-2015 Alessio Garzi <gun101@email.it>
+Copyright (C) 2013-2015 Francesco Minà <mina.francesco@gmail.com>
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License as
@@ -25,11 +26,17 @@ Boston, MA 02111-1307, USA.
 #define GUAKE_PATH "/org/guake/RemoteControl"
 #define GUAKE_IFACE "org.guake.RemoteControl"
 
+DBusGProxy* guake_dbus_init();
 gboolean guake_dbus_send(const gchar*,const gchar*);
+gboolean guake_dbus_send_intparam_with_string_return(const gchar*,gint32,gchar**);
+gboolean guake_dbus_send_noparams_with_integer_return(const gchar*,gint32*);
 gboolean guake_dbus_send_noparam(const gchar*);
 
 gboolean guake_show();
 gboolean guake_newtab();
+gboolean guake_gettabcount(gint32*);
+gboolean guake_getgtktabname(guint,gchar**);
+gboolean guake_selecttab(const gchar*);
 gboolean guake_renamecurrenttab(const gchar*);
 gboolean guake_executecommand(const char*);
 
